/*
 * Plugin Created by ReadMe <glenjamin[at]gmail.com>
 * Can usually be found in irc://irc.moznet.org/chatzilla
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * (Taken from http://people.freebsd.org/~phk/)
 * ReadMe <glenjamin[at]gmail.com> wrote this file. As long as you retain this
 * notice you can do whatever you want with this stuff. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return
 * ----------------------------------------------------------------------------
 */

plugin.id = "WordSwap";

var myprefs = [
["swapsies", "", ""],
];
var swapsies;
var lastSwapID = 0;
var cmdary;

plugin.init =
function init(glob)
{
    /* This function will be called when chatzilla first loads the plugin.
     * We initialize some of the plugin object's properties, define a new
     * command and some new prefs, and print out a message.
     */
    plugin.major = 0;
    plugin.minor = 7;
    plugin.version = plugin.major + "." + plugin.minor;
    plugin.description = "Filters outgoing words"
    plugin.prefary = plugin.prefary.concat(myprefs);

}

plugin.disable =
function disable()
{

    client.commandManager.removeCommands(plugin.commands);

    saveSwaps();
    delete client.outputFilters.swapsies;
    display(plugin.id+' plugin disabled');

    return true;
}

plugin.enable =
function enable()
{
    cmdary =
    [/* comand name     function               flags            usage*/
     ["swap",           addSwap,               null,            "<regex> <replacement>"],
     ["swap-list",      listSwaps,             null,            ""],
     ["swap-show",      showSwap,              null,            "<swapID>"],
     ["swap-find",      listSwaps,             null,            "<regex>"],
     ["swap-del",       delSwap,               null,            "<swapID>"],
     ["swap-clear",     clearSwaps,            null,            ""],
     ["swap-edit",      editSwap,              null,            "<swapID> <part> <new-value>"],
    ]

    cmdary.stringBundle = client.messageManager.addBundle(plugin.cwd+"default.properties");
    client.commandManager.argTypes.__aliasTypes__(["replacement","newValue"], "rest");
    client.commandManager.argTypes["part"] = parsePart;
    plugin.commands = client.commandManager.defineCommands(cmdary);

    if(plugin.prefs['swapsies']!="") {
        swapsies = eval(plugin.prefs['swapsies']);
    } else {
        swapsies = {};
    }

    lastID();

    if (!("outputFilters" in client))
        client.outputFilters = new Object();

    client.outputFilters['swapsies'] = { func: parseSwaps, enabled: true };

    display(plugin.id+' plugin enabled');

    return true;
}

function parsePart(e,name) {

    var ary = e.unparsedData.match (/(\S+)(?:\s+(.*))$/);
    if (!ary)
        return false;
        
    if (ary[1].toLowerCase().substr(0,5) == 'regex' ||
        ary[1].toLowerCase().substr(0,5) == 'match') {
        e[name] = 'matchstr';
        e['partname'] = 'Regex';
    } else if (ary[1].toLowerCase().substr(0,3) != 'rpl' ||
               ary[1].toLowerCase().substr(0,7) != 'replace') {
        e[name] = 'replacestr';
        e['partname'] = 'Replacement';
    } else
        return false;

    e.unparsedData = arrayHasElementAt(ary, 2) ? ary[2] : "";
    
    return true;
    
}

function lastID() {
    
    for (var swapID in swapsies)
        lastSwapID = (swapsies[swapID].ID > lastSwapID) ? swapsies[swapID].ID : lastSwapID ;

}

function saveSwaps() {

    plugin.prefs['swapsies'] = uneval(swapsies);
    display('Stored Swaps pref updated');

}

function parseSwaps(msg, msgtype) {
try {

    for (var swapID in swapsies) {

        matchregex = new RegExp(swapsies[swapID].matchstr, "g");
        msg = msg.replace(matchregex, swapsies[swapID].replacestr);

    }

    return msg;

} catch(ex) {
    display('ERROR: '+ex,'ERROR');
}
}

function editSwap(e) {

    if(e.swapID in swapsies) {
        swapsies[e.swapID][e.part] = e.newValue;
        display("\x02"+e.partname+"\x02 has been updated to \x02"+e.newValue+"\x02 for Swap with ID \x02"+e.swapID);
    } else
        display("No swaps found with ID \x02"+e.swapID);

}

function listSwaps(e) {

    searchstring = e.regex;
    for (var showSwapID in swapsies) {
        matchstring = swapsies[showSwapID].matchstr;
        
        if(e.commandText == 'swap-list' || matchstring.indexOf(searchstring) != -1) {
            var shown = true;
            dispatch("swap-show", {'swapID': showSwapID});
        }
    }
    if (!shown) display('No Swaps Found');
}

function showSwap(e) {

    if(e.swapID in swapsies)
        display(e.swapID+": Regex matching \x02"+swapsies[e.swapID].matchstr+"\x02 will be replaced with \x02"+swapsies[e.swapID].replacestr);
    else
        display("No swaps found with ID \x02"+e.swapID);
}

function delSwap(e) {

    if(e.swapID in swapsies) {
        var matchstr = swapsies[e.swapID].matchstr
        delete swapsies[e.swapID];
        display("Swap matching \x02"+matchstr+"\x02 has been deleted");
        saveSwaps();
    } else
        display("No swaps found with ID \x02"+e.swapID);
}

function clearSwaps() {

    swapsies = {};
    display('All swaps cleared');
    saveSwaps();

}

function addSwap(e) {

    var matchstr = e.regex;
    var replacestr = e.replacement;
    lastSwapID++;
    var newSwapID = lastSwapID;

    swapsies[newSwapID] = {'matchstr': matchstr, 'replacestr': replacestr};

    display("Regex matching \x02"+matchstr+"\x02 will be replaced with \x02"+replacestr);
    saveSwaps();

}
